import yaml

from .deck import GloomhavenDeck


def read_deck(file_path):
    if file_path is None:
        raise ValueError("Provide valid path")
    with open(file_path) as f:
        deck_dict = yaml.safe_load(f)

    return GloomhavenDeck(**deck_dict)



import os

from jinja2 import Environment, FileSystemLoader

package_directory = os.path.dirname(os.path.abspath(__file__))
templateLoader = FileSystemLoader(
    searchpath=os.path.join(package_directory, "templates")
)
env = Environment(loader=templateLoader)


def render_table(table):
    template = env.get_template("table.html")
    return template.render(table=table)



import random

from .constants import DEFAULT_DECK, MODIFIER_APPLIER, RESET_LIST


class GloomhavenDeck:
    RESET_LIST = RESET_LIST

    def __init__(self, cards=None, name=None):
        self.current_deck = []
        self.name = name
        if cards is None or len(cards) < 1:
            self.card_list = DEFAULT_DECK
        else:
            self.card_list = cards

        if all([card[0] == "c" for card in self.card_list]):
            raise ValueError("Invalid deck: contains all continue cards.")

    def shuffle_deck(self):
        self.empty_deck()
        tmp = random.sample(self.card_list, len(self.card_list))
        for card in tmp:
            self.current_deck.append(card)

    def empty_deck(self):
        self.current_deck = []

    def draw_card(self):
        if not len(self.current_deck):
            self.shuffle_deck()

        card_to_return = self.current_deck.pop()
        if card_to_return in self.RESET_LIST:
            self.shuffle_deck()
        return card_to_return

    def get_attack(self, base_attack):
        drawn_card = self.draw_card()
        attack_dmg = base_attack
        while True:
            if drawn_card[0] == "c":
                attack_dmg = MODIFIER_APPLIER[drawn_card[1:]](attack_dmg)
                drawn_card = self.draw_card()
            else:
                return MODIFIER_APPLIER[drawn_card](attack_dmg)

    def __repr__(self):
        return "\n".join(
            [
                self.name if self.name is not None else "",
                f"Cards: {', '.join(self.card_list)}",
                f"Current Deck: {', '.join(reversed(self.current_deck))}",
            ]
        )

    def __str__(self):
        return self.__repr__()

    def copy(self):
        return GloomhavenDeck(cards=self.card_list)



from .mod_applier import ModDeckApplier, mod_applier_fn

DEFAULT_DECK = [
    "Miss",
    "2x",
    "-2",
    "2",
    *["1"] * 5,
    *["-1"] * 5,
    *["0"] * 6,
]

RESET_LIST = ["Miss", "2x"]

MODIFIER_APPLIER = ModDeckApplier(mod_applier_fn)
MODIFIER_APPLIER["Miss"] = lambda x: 0
MODIFIER_APPLIER["2x"] = lambda x: 2 * x



class ModDeckApplier(dict):
    def __init__(self, default_fn):
        self.default_fn = default_fn

    def __missing__(self, k):
        return self.default_fn(k)


def mod_applier_fn(k):
    def apply_mod(x):
        return x + int(k) if (x + int(k)) >= 0 else 0

    return apply_mod



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gloomhaven']

package_data = \
{'': ['*'], 'gloomhaven': ['templates/*']}

install_requires = \
['Jinja2>=3.0.1,<4.0.0', 'PyYAML>=5.4.1,<6.0.0']

extras_require = \
{'analyze': ['pandas>=1.3.3,<2.0.0',
             'dask[distributed]>=2021.9.0,<2022.0.0',
             'matplotlib>=3.4.3,<4.0.0',
             'jupyter>=1.0.0,<2.0.0']}

setup_kwargs = {
    'name': 'gloomhaven',
    'version': '0.3.0',
    'description': 'Probability evaluation of Gloomhaven attack modifier decks',
    'long_description': None,
    'author': 'Zachary Coleman',
    'author_email': 'zacharywcoleman@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/zachcoleman/gloomhaven',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7.1,<4',
}


setup(**setup_kwargs)
